"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateErrors = void 0;
var CertificateErrors;
(function (CertificateErrors) {
    let Issuance;
    (function (Issuance) {
        class CertificateAlreadyIssued extends Error {
            constructor(internalCertificateId) {
                super(`Issuance for internalCertificateId: ${internalCertificateId} failed. Certificate already issued.`);
            }
        }
        Issuance.CertificateAlreadyIssued = CertificateAlreadyIssued;
    })(Issuance = CertificateErrors.Issuance || (CertificateErrors.Issuance = {}));
    let Transfer;
    (function (Transfer) {
        class CertificateNotIssued extends Error {
            constructor(internalCertificateId) {
                super(`Transfer for internalCertificateId: ${internalCertificateId} failed. Certificate was not issued.`);
            }
        }
        Transfer.CertificateNotIssued = CertificateNotIssued;
        class FromZeroAddress extends Error {
            constructor(internalCertificateId) {
                super(`Transfer for internalCertificateId: ${internalCertificateId} failed. Transfer is from zero address(0x0).`);
            }
        }
        Transfer.FromZeroAddress = FromZeroAddress;
        class ToZeroAddress extends Error {
            constructor(internalCertificateId) {
                super(`Transfer for internalCertificateId: ${internalCertificateId} failed. Transfer is to zero address(0x0).`);
            }
        }
        Transfer.ToZeroAddress = ToZeroAddress;
        class NotEnoughBalance extends Error {
            constructor(internalCertificateId, address) {
                super(`Transfer for: ${internalCertificateId} failed. Address: ${address} has not enough balance.`);
            }
        }
        Transfer.NotEnoughBalance = NotEnoughBalance;
    })(Transfer = CertificateErrors.Transfer || (CertificateErrors.Transfer = {}));
    let Claim;
    (function (Claim) {
        class CertificateNotIssued extends Error {
            constructor(internalCertificateId) {
                super(`Claim for internalCertificateId: ${internalCertificateId} failed. Certificate was not issued.`);
            }
        }
        Claim.CertificateNotIssued = CertificateNotIssued;
        class ForZeroAddress extends Error {
            constructor(internalCertificateId) {
                super(`Claim for internalCertificateId: ${internalCertificateId} failed. Transfer is for zero address(0x0).`);
            }
        }
        Claim.ForZeroAddress = ForZeroAddress;
        class NotEnoughBalance extends Error {
            constructor(internalCertificateId, address) {
                super(`Claim for: ${internalCertificateId} failed. Address: ${address} has not enough balance.`);
            }
        }
        Claim.NotEnoughBalance = NotEnoughBalance;
    })(Claim = CertificateErrors.Claim || (CertificateErrors.Claim = {}));
    class BatchError extends Error {
        constructor(originError) {
            super(`Batch operation failed. Failing operation message: ${originError.message}`);
        }
    }
    CertificateErrors.BatchError = BatchError;
})(CertificateErrors = exports.CertificateErrors || (exports.CertificateErrors = {}));
//# sourceMappingURL=errors.js.map