import { IEvent } from '@nestjs/cqrs';
import { IClaimCommand, IIssueCommand, ITransferCommand } from '../../types';
export declare enum CertificateEventType {
    Issued = "Issued",
    Transferred = "Transferred",
    Claimed = "Claimed",
    IssuancePersisted = "IssuancePersisted",
    TransferPersisted = "TransferPersisted",
    ClaimPersisted = "ClaimPersisted",
    PersistError = "PersistError"
}
export interface ICertificateEvent {
    type: CertificateEventType;
    version: number;
    internalCertificateId: number;
    payload: unknown;
    createdAt: Date;
}
export declare class CertificateIssuedEvent<MetadataType = unknown> implements IEvent {
    readonly internalCertificateId: number;
    readonly payload: IIssueCommand<MetadataType>;
    readonly type = CertificateEventType.Issued;
    readonly version = 1;
    readonly createdAt: Date;
    constructor(internalCertificateId: number, payload: IIssueCommand<MetadataType>);
}
export declare class CertificateTransferredEvent implements IEvent {
    readonly internalCertificateId: number;
    readonly payload: ITransferCommand;
    readonly type = CertificateEventType.Transferred;
    readonly version = 1;
    readonly createdAt: Date;
    constructor(internalCertificateId: number, payload: ITransferCommand);
}
export declare class CertificateClaimedEvent implements IEvent {
    readonly internalCertificateId: number;
    readonly payload: IClaimCommand;
    readonly type = CertificateEventType.Claimed;
    readonly version = 1;
    readonly createdAt: Date;
    constructor(internalCertificateId: number, payload: IClaimCommand);
}
export declare class CertificateIssuancePersistedEvent implements IEvent {
    readonly internalCertificateId: number;
    readonly payload: {
        blockchainCertificateId: number;
    };
    readonly type = CertificateEventType.IssuancePersisted;
    readonly version = 1;
    readonly createdAt: Date;
    constructor(internalCertificateId: number, payload: {
        blockchainCertificateId: number;
    });
}
export declare class CertificateTransferPersistedEvent implements IEvent {
    readonly internalCertificateId: number;
    readonly payload: {};
    readonly type = CertificateEventType.TransferPersisted;
    readonly version = 1;
    readonly createdAt: Date;
    constructor(internalCertificateId: number, payload: {});
}
export declare class CertificateClaimPersistedEvent implements IEvent {
    readonly internalCertificateId: number;
    readonly payload: {};
    readonly type = CertificateEventType.ClaimPersisted;
    readonly version = 1;
    readonly createdAt: Date;
    constructor(internalCertificateId: number, payload: {});
}
export declare class CertificatePersistErrorEvent implements IEvent {
    internalCertificateId: number;
    payload: {
        errorMessage: string;
    };
    readonly type = CertificateEventType.PersistError;
    readonly version = 1;
    readonly createdAt: Date;
    constructor(internalCertificateId: number, payload: {
        errorMessage: string;
    });
}
