"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OffChainCertificateService = void 0;
const common_1 = require("@nestjs/common");
const cqrs_1 = require("@nestjs/cqrs");
const Certificate_events_1 = require("./events/Certificate.events");
const certificate_aggregate_1 = require("./certificate.aggregate");
const errors_1 = require("./errors");
const repository_keys_1 = require("./repositories/repository.keys");
const CertificateEvent_service_1 = require("./repositories/CertificateEvent/CertificateEvent.service");
let OffChainCertificateService = class OffChainCertificateService {
    constructor(certCommandRepo, certEventRepo, certificateEventService, eventBus, readModelRepo) {
        this.certCommandRepo = certCommandRepo;
        this.certEventRepo = certEventRepo;
        this.certificateEventService = certificateEventService;
        this.eventBus = eventBus;
        this.readModelRepo = readModelRepo;
    }
    async getAll(options = {}) {
        const certificates = await this.readModelRepo.getAll(options);
        return certificates;
    }
    async getById(id) {
        const certificate = await this.readModelRepo.getByInternalCertificateId(id);
        return certificate;
    }
    async issue(params) {
        const command = {
            ...params,
            fromTime: Math.round(params.fromTime.getTime() / 1000),
            toTime: Math.round(params.toTime.getTime() / 1000)
        };
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = new Certificate_events_1.CertificateIssuedEvent(await this.generateInternalCertificateId(), command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
        return event.internalCertificateId;
    }
    async claim(command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = new Certificate_events_1.CertificateClaimedEvent(command.certificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async transfer(command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = new Certificate_events_1.CertificateTransferredEvent(command.certificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async issuePersisted(internalCertificateId, command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = new Certificate_events_1.CertificateIssuancePersistedEvent(internalCertificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async claimPersisted(internalCertificateId, command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = new Certificate_events_1.CertificateClaimPersistedEvent(internalCertificateId, {});
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async transferPersisted(internalCertificateId, command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = new Certificate_events_1.CertificateTransferPersistedEvent(internalCertificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate);
    }
    async persistError(internalCertificateId, command) {
        const savedCommand = await this.certCommandRepo.save({ payload: command });
        const event = new Certificate_events_1.CertificatePersistErrorEvent(internalCertificateId, command);
        const aggregate = await this.createAggregate([event]);
        await this.propagate(event, savedCommand, aggregate, command.errorMessage);
    }
    async batchIssue(originalCertificates) {
        const commands = originalCertificates.map((c) => ({
            ...c,
            fromTime: Math.round(c.fromTime.getTime() / 1000),
            toTime: Math.round(c.toTime.getTime() / 1000)
        }));
        await this.validateBatchIssue(commands);
        const certs = [];
        for (const certificate of originalCertificates) {
            const certificateId = await this.issue(certificate);
            certs.push(certificateId);
        }
        return certs;
    }
    async batchClaim(commands) {
        await this.validateBatchClaim(commands);
        for (const command of commands) {
            await this.claim(command);
        }
    }
    async batchTransfer(commands) {
        await this.validateBatchTransfer(commands);
        for (const command of commands) {
            await this.transfer(command);
        }
    }
    async validateBatchIssue(commands) {
        try {
            commands.forEach(async (c) => {
                certificate_aggregate_1.CertificateAggregate.fromEvents([
                    new Certificate_events_1.CertificateIssuedEvent(await this.generateInternalCertificateId(), c)
                ]);
            });
        }
        catch (err) {
            throw new errors_1.CertificateErrors.BatchError(err);
        }
    }
    async validateBatchClaim(commands) {
        const grouped = this.groupCommandsByCertificate(commands);
        try {
            for (const certificateId in grouped) {
                const events = grouped[certificateId].map((c) => {
                    return new Certificate_events_1.CertificateClaimedEvent(parseInt(certificateId), c);
                });
                await this.createAggregate(events);
            }
        }
        catch (err) {
            throw new errors_1.CertificateErrors.BatchError(err);
        }
    }
    async validateBatchTransfer(commands) {
        const grouped = this.groupCommandsByCertificate(commands);
        try {
            for (const certificateId in grouped) {
                const events = grouped[certificateId].map((c) => {
                    return new Certificate_events_1.CertificateTransferredEvent(parseInt(certificateId), c);
                });
                await this.createAggregate(events);
            }
        }
        catch (err) {
            throw new errors_1.CertificateErrors.BatchError(err);
        }
    }
    async createAggregate(events) {
        const aggregate = certificate_aggregate_1.CertificateAggregate.fromEvents([
            ...(await this.certEventRepo.getByInternalCertificateId(events[0].internalCertificateId)),
            ...events
        ]);
        return aggregate;
    }
    async generateInternalCertificateId() {
        const numberOfCertificates = await this.certEventRepo.getNumberOfCertificates();
        return numberOfCertificates + 1;
    }
    async propagate(event, command, aggregate, errorMessage) {
        await this.eventBus.publish(event);
        const savedEvent = await this.certificateEventService.save(event, command.id);
        await this.readModelRepo.save(aggregate.getCertificate());
        await this.certEventRepo.updateAttempt({
            eventId: savedEvent.id,
            error: errorMessage
        });
    }
    groupCommandsByCertificate(commands) {
        const certificates = {};
        commands.forEach((command) => {
            const exists = certificates[command.certificateId];
            exists
                ? (certificates[command.certificateId] = [...exists, command])
                : (certificates[command.certificateId] = [command]);
        });
        return certificates;
    }
};
OffChainCertificateService = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(repository_keys_1.CERTIFICATE_COMMAND_REPOSITORY)),
    __param(1, common_1.Inject(repository_keys_1.CERTIFICATE_EVENT_REPOSITORY)),
    __param(4, common_1.Inject(repository_keys_1.CERTIFICATE_READ_MODEL_REPOSITORY)),
    __metadata("design:paramtypes", [Object, Object, CertificateEvent_service_1.CertificateEventService,
        cqrs_1.EventBus, Object])
], OffChainCertificateService);
exports.OffChainCertificateService = OffChainCertificateService;
//# sourceMappingURL=offchain-certificate.service.js.map