"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateEventInMemoryRepository = void 0;
const common_1 = require("@nestjs/common");
const Certificate_events_1 = require("../../events/Certificate.events");
const blockchain_synchronize_const_1 = require("../../synchronize/blockchain-synchronize.const");
let CertificateEventInMemoryRepository = class CertificateEventInMemoryRepository {
    constructor() {
        this.eventDb = [];
        this.attemptDb = {};
    }
    async getAll() {
        return this.eventDb;
    }
    async getByInternalCertificateId(internalCertId) {
        return this.eventDb.filter((e) => e.internalCertificateId === internalCertId);
    }
    async save(event, commandId) {
        const entity = {
            internalCertificateId: event.internalCertificateId,
            type: event.type,
            version: event.version,
            payload: event.payload,
            createdAt: event.createdAt,
            commandId: commandId,
            id: this.eventDb.length + 1
        };
        this.eventDb.push(entity);
        return entity;
    }
    async updateAttempt({ eventId, error }) {
        const synchronizationAttempt = await this.attemptDb[eventId];
        if (!synchronizationAttempt) {
            throw new Error('Synchronization attempt does not exist');
        }
        synchronizationAttempt.attemptsCount = synchronizationAttempt.attemptsCount + 1;
        synchronizationAttempt.error = error ?? null;
        return synchronizationAttempt;
    }
    async getAllNotProcessed() {
        const attempts = Object.values(this.attemptDb).filter((a) => {
            return ((!a.error && a.attemptsCount === 0) ||
                (a.error && a.attemptsCount < blockchain_synchronize_const_1.MAX_SYNCHRONIZATION_ATTEMPTS_FOR_EVENT));
        });
        const events = attempts.map((a) => {
            return this.eventDb.find((e) => e.id === a.eventId);
        });
        return events;
    }
    async getOne(eventId) {
        const event = this.eventDb.find((e) => e.id === eventId);
        if (!event) {
            throw new common_1.NotFoundException();
        }
        return event;
    }
    async getNumberOfCertificates() {
        return this.eventDb.filter((e) => e.type === Certificate_events_1.CertificateEventType.Issued).length;
    }
};
CertificateEventInMemoryRepository = __decorate([
    common_1.Injectable()
], CertificateEventInMemoryRepository);
exports.CertificateEventInMemoryRepository = CertificateEventInMemoryRepository;
//# sourceMappingURL=CertificateEventInMemory.repository.js.map