"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateEventPostgresRepository = void 0;
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const typeorm_2 = require("typeorm");
const CertificateEvent_entity_1 = require("./CertificateEvent.entity");
const Certificate_events_1 = require("../../events/Certificate.events");
const CertificateSynchronizationAttempt_entity_1 = require("./CertificateSynchronizationAttempt.entity");
const blockchain_synchronize_const_1 = require("../../synchronize/blockchain-synchronize.const");
let CertificateEventPostgresRepository = class CertificateEventPostgresRepository {
    constructor(repository, synchronizationAttemptRepository) {
        this.repository = repository;
        this.synchronizationAttemptRepository = synchronizationAttemptRepository;
    }
    async getAll() {
        return await this.repository.find();
    }
    async getByInternalCertificateId(internalCertId) {
        return await this.repository.find({
            where: {
                internalCertificateId: internalCertId
            }
        });
    }
    async save(event, commandId, txManager) {
        const repository = txManager
            ? txManager.getRepository(CertificateEvent_entity_1.CertificateEventEntity)
            : this.repository;
        return await repository.save({
            internalCertificateId: event.internalCertificateId,
            type: event.type,
            version: event.version,
            payload: event.payload,
            createdAt: event.createdAt,
            commandId: commandId
        });
    }
    async updateAttempt({ eventId, error }) {
        const synchronizationAttempt = await this.synchronizationAttemptRepository.findOne(eventId);
        if (!synchronizationAttempt) {
            throw new Error('Synchronization attempt does not exist');
        }
        synchronizationAttempt.attemptsCount = synchronizationAttempt.attemptsCount + 1;
        synchronizationAttempt.error = error ?? null;
        await this.synchronizationAttemptRepository.update(synchronizationAttempt.eventId, synchronizationAttempt);
        return synchronizationAttempt;
    }
    async getAllNotProcessed() {
        const query = this.repository
            .createQueryBuilder('event')
            .select([
            'event.id',
            'event.internalCertificateId',
            'event.createdAt',
            'event.commandId',
            'event.type',
            'event.version',
            'event.payload'
        ])
            .leftJoinAndSelect(CertificateSynchronizationAttempt_entity_1.CertificateSynchronizationAttemptEntity, 'attempt', 'attempt.eventId = event.id')
            .where('(attempt.error IS NULL AND attempt.attempts_count = 0)')
            .orWhere(`(attempt.error IS NOT NULL AND attempt.attempts_count < ${blockchain_synchronize_const_1.MAX_SYNCHRONIZATION_ATTEMPTS_FOR_EVENT})`);
        return (await query.getMany());
    }
    async getOne(eventId) {
        const event = await this.repository.findOne(eventId);
        if (!event) {
            throw new common_1.NotFoundException();
        }
        return event;
    }
    async getNumberOfCertificates() {
        const [certs, count] = await this.repository.findAndCount({
            where: {
                type: Certificate_events_1.CertificateEventType.Issued
            }
        });
        return count;
    }
};
CertificateEventPostgresRepository = __decorate([
    common_1.Injectable(),
    __param(0, typeorm_1.InjectRepository(CertificateEvent_entity_1.CertificateEventEntity)),
    __param(1, typeorm_1.InjectRepository(CertificateSynchronizationAttempt_entity_1.CertificateSynchronizationAttemptEntity)),
    __metadata("design:paramtypes", [typeorm_2.Repository,
        typeorm_2.Repository])
], CertificateEventPostgresRepository);
exports.CertificateEventPostgresRepository = CertificateEventPostgresRepository;
//# sourceMappingURL=CertificateEventPostgres.repository.js.map