"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SynchronizeManager = void 0;
const claim_persist_handler_1 = require("./claim-persist.handler");
const transfer_persist_handler_1 = require("./transfer-persist.handler");
const common_1 = require("@nestjs/common");
const issue_persist_handler_1 = require("./issue-persist.handler");
let SynchronizeManager = class SynchronizeManager {
    constructor(claimPersistHandler, issuancePersistHandler, transferPersistHandler) {
        this.claimPersistHandler = claimPersistHandler;
        this.issuancePersistHandler = issuancePersistHandler;
        this.transferPersistHandler = transferPersistHandler;
        this.processors = [];
        this.processors = [claimPersistHandler, transferPersistHandler, issuancePersistHandler];
    }
    async handle(event) {
        const applicableProcessor = this.processors.find((handler) => handler.canHandle(event));
        if (!applicableProcessor) {
            throw new Error(`Cannot find handler to process event of type: ${event.type}`);
        }
        return await applicableProcessor.handle(event);
    }
    async handleBatch(events) {
        let processors = [
            { handler: this.claimPersistHandler, events: [] },
            { handler: this.transferPersistHandler, events: [] },
            { handler: this.issuancePersistHandler, events: [] }
        ];
        for (let event of events) {
            processors = processors.map((processor) => {
                if (processor.handler.canHandle(event)) {
                    return {
                        events: [...processor.events, event],
                        handler: processor.handler
                    };
                }
                return processor;
            });
        }
        const failedCertificateIds = [];
        for (let processor of processors) {
            const eventsBatch = processor.events;
            const result = await processor.handler.handleBatch(eventsBatch);
            failedCertificateIds.push(...result.failedCertificateIds);
        }
        return { failedCertificateIds };
    }
};
SynchronizeManager = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [claim_persist_handler_1.ClaimPersistHandler,
        issue_persist_handler_1.IssuePersistHandler,
        transfer_persist_handler_1.TransferPersistHandler])
], SynchronizeManager);
exports.SynchronizeManager = SynchronizeManager;
//# sourceMappingURL=synchronize.manager.js.map