"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransferPersistHandler = void 0;
const types_1 = require("../../../onchain-certificate/types");
const onchain_certificate_service_1 = require("../../../onchain-certificate/onchain-certificate.service");
const Certificate_events_1 = require("../../events/Certificate.events");
const offchain_certificate_service_1 = require("../../offchain-certificate.service");
const common_1 = require("@nestjs/common");
const lodash_1 = require("lodash");
const batch_configuration_1 = require("../strategies/batch/batch.configuration");
let TransferPersistHandler = class TransferPersistHandler {
    constructor(certificateService, offchainCertificateService, batchConfiguration) {
        this.certificateService = certificateService;
        this.offchainCertificateService = offchainCertificateService;
        this.batchConfiguration = batchConfiguration;
    }
    canHandle(event) {
        return event.type === Certificate_events_1.CertificateEventType.Transferred;
    }
    async handle(event) {
        const transferredEvent = event;
        const result = await this.certificateService.transfer(transferredEvent.payload);
        if (result.success) {
            await this.offchainCertificateService.transferPersisted(event.internalCertificateId, {});
            return { success: true };
        }
        else {
            await this.offchainCertificateService.persistError(event.internalCertificateId, {
                errorMessage: `[${result.statusCode}] ${result.message}`
            });
            return { success: false };
        }
    }
    async synchronizeBatchBlock(events) {
        const transferredEvents = events;
        const result = await this.certificateService.batchTransfer(transferredEvents.map((event) => event.payload));
        const failedCertificateIds = await Promise.all(events.map(async (event) => {
            if (result.success) {
                await this.offchainCertificateService.transferPersisted(event.internalCertificateId, {});
            }
            else {
                await this.offchainCertificateService.persistError(event.internalCertificateId, {
                    errorMessage: `[${result.statusCode}] ${result.message}`
                });
                return event.internalCertificateId;
            }
        }));
        return { failedCertificateIds: lodash_1.compact(failedCertificateIds) };
    }
    async handleBatch(events) {
        const eventsBlocks = lodash_1.chunk(events, this.batchConfiguration.transferBatchSize);
        const failedCertificateIds = [];
        for (const eventsBlock of eventsBlocks) {
            const nonFailedEvents = eventsBlock.filter((event) => !failedCertificateIds.includes(event.internalCertificateId));
            const result = await this.synchronizeBatchBlock(nonFailedEvents);
            failedCertificateIds.push(...result.failedCertificateIds);
        }
        return { failedCertificateIds };
    }
};
TransferPersistHandler = __decorate([
    common_1.Injectable(),
    __param(0, common_1.Inject(types_1.ONCHAIN_CERTIFICATE_SERVICE_TOKEN)),
    __param(2, common_1.Inject(batch_configuration_1.BATCH_CONFIGURATION_TOKEN)),
    __metadata("design:paramtypes", [onchain_certificate_service_1.OnChainCertificateService,
        offchain_certificate_service_1.OffChainCertificateService, Object])
], TransferPersistHandler);
exports.TransferPersistHandler = TransferPersistHandler;
//# sourceMappingURL=transfer-persist.handler.js.map