"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BatchSynchronizeStrategy = void 0;
const common_1 = require("@nestjs/common");
const synchronize_manager_1 = require("../../handlers/synchronize.manager");
const events_batch_queue_1 = require("./events.batch-queue");
const Certificate_events_1 = require("../../../events/Certificate.events");
let BatchSynchronizeStrategy = class BatchSynchronizeStrategy {
    constructor(synchronizeManager) {
        this.synchronizeManager = synchronizeManager;
    }
    async synchronize(events) {
        const failedCertificateIds = [];
        const hasCertificateFailed = (certificateId) => failedCertificateIds.includes(certificateId);
        const eventsBatchQueue = new events_batch_queue_1.EventsBatchQueue(events);
        while (!eventsBatchQueue.isEmpty()) {
            const batch = eventsBatchQueue.popBatch();
            const issueEvents = batch
                .filter((e) => e.type === Certificate_events_1.CertificateEventType.Issued)
                .filter((e) => !hasCertificateFailed(e.internalCertificateId));
            if (issueEvents.length) {
                const issueResult = await this.synchronizeManager.handleBatch(issueEvents);
                failedCertificateIds.push(...issueResult.failedCertificateIds);
                eventsBatchQueue.removeEventsForCertificateIds(issueResult.failedCertificateIds);
            }
            const transferEvents = batch
                .filter((e) => e.type === Certificate_events_1.CertificateEventType.Transferred)
                .filter((e) => !hasCertificateFailed(e.internalCertificateId));
            if (transferEvents.length) {
                const transferResult = await this.synchronizeManager.handleBatch(transferEvents);
                eventsBatchQueue.removeEventsForCertificateIds(transferResult.failedCertificateIds);
            }
            const claimEvents = batch
                .filter((e) => e.type === Certificate_events_1.CertificateEventType.Claimed)
                .filter((e) => !hasCertificateFailed(e.internalCertificateId));
            if (claimEvents.length) {
                const claimResult = await this.synchronizeManager.handleBatch(claimEvents);
                failedCertificateIds.push(...claimResult.failedCertificateIds);
                eventsBatchQueue.removeEventsForCertificateIds(claimResult.failedCertificateIds);
            }
        }
    }
};
BatchSynchronizeStrategy = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [synchronize_manager_1.SynchronizeManager])
], BatchSynchronizeStrategy);
exports.BatchSynchronizeStrategy = BatchSynchronizeStrategy;
//# sourceMappingURL=batch-synchronize.strategy.js.map