"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventsBatchQueue = void 0;
const lodash_1 = require("lodash");
class EventsBatchQueue {
    constructor(events) {
        this.eventsToProcess = {};
        const sortedEvents = events.sort((a, b) => a.createdAt.getTime() - b.createdAt.getTime());
        this.eventsToProcess = lodash_1.groupBy(sortedEvents, 'internalCertificateId');
    }
    popBatch() {
        const eventsToProcess = [];
        Object.entries(this.eventsToProcess).forEach(([certificateId, eventsForCertificate]) => {
            const [firstEvent, ...otherEvents] = eventsForCertificate;
            if (firstEvent) {
                this.eventsToProcess[certificateId] = otherEvents;
                eventsToProcess.push(firstEvent);
            }
            else {
                this.removeEventsForCertificateIds([Number(certificateId)]);
            }
        });
        return eventsToProcess;
    }
    isEmpty() {
        return lodash_1.flatten(Object.values(this.eventsToProcess)).length === 0;
    }
    removeEventsForCertificateIds(internalCertificateIds) {
        internalCertificateIds.forEach((internalCertificateId) => {
            delete this.eventsToProcess[internalCertificateId];
        });
    }
}
exports.EventsBatchQueue = EventsBatchQueue;
//# sourceMappingURL=events.batch-queue.js.map