"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.OnChainCertificateService = void 0;
const issuer_api_1 = require("@energyweb/issuer-api");
const common_1 = require("@nestjs/common");
const cqrs_1 = require("@nestjs/cqrs");
const bull_1 = require("@nestjs/bull");
const types_1 = require("./types");
const blockchain_actions_processor_1 = require("./blockchain-actions.processor");
const jobOptions = {
    removeOnComplete: true,
    removeOnFail: true
};
let OnChainCertificateService = class OnChainCertificateService {
    constructor(queryBus, blockchainActionsQueue) {
        this.queryBus = queryBus;
        this.blockchainActionsQueue = blockchainActionsQueue;
    }
    async getAll(options = {}) {
        const certificates = await this.queryBus.execute(new issuer_api_1.GetAllCertificatesQuery(options));
        return certificates.map((c) => this.mapCertificate(c));
    }
    async getById(id) {
        const certificate = await this.queryBus.execute(new issuer_api_1.GetCertificateQuery(id));
        return certificate ? this.mapCertificate(certificate) : null;
    }
    async issue(params) {
        const command = {
            ...params,
            fromTime: Math.round(params.fromTime.getTime() / 1000),
            toTime: Math.round(params.toTime.getTime() / 1000)
        };
        const job = await this.blockchainActionsQueue.add({
            payload: command,
            type: types_1.BlockchainActionType.Issuance
        }, jobOptions);
        const result = await this.waitForJobResult(job);
        return this.mapCertificate(result);
    }
    async claim(command) {
        const job = await this.blockchainActionsQueue.add({
            payload: command,
            type: types_1.BlockchainActionType.Claim
        }, jobOptions);
        const result = await this.waitForJobResult(job);
        return result;
    }
    async transfer(command) {
        const job = await this.blockchainActionsQueue.add({
            payload: command,
            type: types_1.BlockchainActionType.Transfer
        }, jobOptions);
        const result = await this.waitForJobResult(job);
        return result;
    }
    async batchIssue(originalCertificates) {
        if (originalCertificates.length === 0) {
            return [];
        }
        const certificates = originalCertificates.map((certificate) => ({
            ...certificate,
            fromTime: Math.round(certificate.fromTime.getTime() / 1000),
            toTime: Math.round(certificate.toTime.getTime() / 1000)
        }));
        const job = await this.blockchainActionsQueue.add({
            payload: {
                certificates
            },
            type: types_1.BlockchainActionType.BatchIssuance
        }, jobOptions);
        const result = await this.waitForJobResult(job);
        return result;
    }
    async batchClaim(command) {
        if (command.length === 0) {
            return {
                success: true
            };
        }
        const job = await this.blockchainActionsQueue.add({
            payload: {
                claims: command
            },
            type: types_1.BlockchainActionType.BatchClaim
        }, jobOptions);
        const result = await this.waitForJobResult(job);
        return result;
    }
    async batchTransfer(command) {
        if (command.length === 0) {
            return {
                success: true
            };
        }
        const job = await this.blockchainActionsQueue.add({
            payload: {
                transfers: command
            },
            type: types_1.BlockchainActionType.BatchTransfer
        }, jobOptions);
        const result = await this.waitForJobResult(job);
        return result;
    }
    mapCertificate(certificate) {
        return {
            ...certificate,
            claims: certificate.claims ?? [],
            metadata: certificate.metadata !== '' ? JSON.parse(certificate.metadata) : null
        };
    }
    async waitForJobResult(job) {
        try {
            return await job.finished();
        }
        catch (e) {
            const error = new Error(e.message);
            error.stack =
                '[Hidden job error stack trace]. Please refer to logger logs for error stacktrace';
            throw error;
        }
    }
};
OnChainCertificateService = __decorate([
    common_1.Injectable(),
    __param(1, bull_1.InjectQueue(blockchain_actions_processor_1.blockchainQueueName)),
    __metadata("design:paramtypes", [cqrs_1.QueryBus, Object])
], OnChainCertificateService);
exports.OnChainCertificateService = OnChainCertificateService;
//# sourceMappingURL=onchain-certificate.service.js.map