"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransactionPollService = void 0;
const common_1 = require("@nestjs/common");
const issuer_api_1 = require("@energyweb/issuer-api");
const cqrs_1 = require("@nestjs/cqrs");
const pollDelay = 3000;
const maxRetries = 100;
let TransactionPollService = class TransactionPollService {
    constructor(queryBus) {
        this.queryBus = queryBus;
        this.processedTransactions = {};
    }
    async waitForNewCertificates(txHash) {
        return await this.poll(txHash, async (hash) => {
            const result = await this.queryBus.execute(new issuer_api_1.GetCertificateByTxHashQuery(hash));
            if (result.length === 0) {
                return new Error('Could not find certificate');
            }
            return result;
        });
    }
    async waitForTransaction(txHash) {
        await this.poll(txHash, async (hash) => {
            const isPresent = hash in this.processedTransactions;
            if (isPresent) {
                delete this.processedTransactions[hash];
            }
            else {
                return new Error('Transaction was not registered');
            }
        });
    }
    async saveTransactionAsProcessed(txHash) {
        this.processedTransactions[txHash] = true;
    }
    async poll(txHash, cb, tryCount = 0) {
        const result = await cb(txHash);
        if (result instanceof Error) {
            if (tryCount === maxRetries) {
                throw new Error(`Unable to query for transaction ${txHash} for ${maxRetries * pollDelay}ms. Latest reason: ${result.message}`);
            }
            await new Promise((resolve) => setTimeout(resolve, pollDelay));
            return this.poll(txHash, cb, tryCount + 1);
        }
        return result;
    }
};
TransactionPollService = __decorate([
    common_1.Injectable(),
    __metadata("design:paramtypes", [cqrs_1.QueryBus])
], TransactionPollService);
exports.TransactionPollService = TransactionPollService;
//# sourceMappingURL=transaction-poll.service.js.map